//==============================================================================
//
//
//    Receptor para ht6p20b - por captura bit a bit aps sincronizao
//    (sem uso de interrupes)
//
//    Autor: Claudio Lrios   12/07/2015
//
//
//    Permite o modo de operao da sada em pulso ou reteno,  e tambm modo
//    normal ou invertido, selecionado pelo usurio de forma individual.
//    Comente/descomente em 'configurao do usurio' e recompile o arquivo
//    usando o  compilador CCS C.
//
//    Este arquivo  parte integrante do blog Larios.Tecnologia.WS
//    Uso didtico e sujeito a bugs no identificados ainda.
//
//
//==============================================================================

#include <12F675.h>
#device adc=8
#use delay(clock=4000000)
#ignore_warnings 203,202  //desconsidera mensagem de 'condition always true'
#fuses NOWDT,INTRC_IO, NOCPD, NOPROTECT, NOMCLR, NOPUT, BROWNOUT
#ZERO_RAM
#use fast_io(a)
//bytes
#byte tris=0x85
#byte gpio=0x05
//bits
#bit LED1=0x05.1 //saida canal 1 no pino 6
#bit LED2=0x05.2 //saida canal 2 no pino 5
#bit LED3=0X05.4 //saida canal 3 no pino 3
//bit
#bit LEDG=0x05.5//led learn  pino 2
#bit LRN=0X05.3//boto de gravao/apagamento  pino 4
#bit RF  =0x05.0 //entrada do sinal de RF pino 7
#bit  t0if=0x0b.2//flag over flow timer 0
//defines
#define NBIT 28 // numero de bits
#define apagar 70 //tempo com boto LRN apertado para apagar eeprom
#define T_MAX  250 //tempo maximo em modo de gravao de controles (15 seg)
#define T_basico 500 //tempo basico (pulso inicial da transmisso)
#define v_tempo_acionado 4000 //tempo das saidas acionadas em modo pulso
#define t_intervalo 400 // intervalo entre transmisses (400 a 5000)
#define tris_led  0b001001; //trisio para acender led prog
//#define tris_lrn  0b001001; //trisio para acender led prog



//variavis globais
int32 buffer=0; //buffer de dados a receber
int32 bufferx=0;//buffer de copia da transmisso anterior
int16 time_led1;//contador para desligamento de saidas acionadas
int16 time_led2;//contador para desligamento de saidas acionadas
int16 time_led3;//contador para desligamento de saidas acionadas
int16 timerx=0;
int1  fled1,fled2,fled3;//flags auxiliares
void testa_lrn(); //prototipo

//==============================================================================
//          CONFIGURAES DO USURIO
//==============================================================================
//  COMENTE (//) PARA MODO RETENO E DESCOMENTE PARA MODO PULSO

   #define  led1_pulso
   #define  led2_pulso
   #define  led3_pulso


//==============================================================================
//  COMENTE (//) PARA MODO NORMAL E DESCOMENTE PARA MODO INVERTIDO

 //  #define  led1_invertido
 //  #define  led2_invertido
 //  #define  led3_invertido


//==============================================================================
//          Rotina para captura de bits enviados pelo tx (ht6p20b)
//==============================================================================

int1 get_htp(void) {
   int8  x=0;
   buffer=0;
   while(!RF){
   testa_lrn();
   if(!++timerx)  return false;
   }

  if(timerx>t_intervalo) { //  intervalo entre transmisses (23te x pulso basico)

      for(x=0;x<NBIT-1;x++){

         while(RF) {
          testa_lrn();
         }
         delay_us(T_basico/2);//meio t_basico

         if(RF){
         return FALSE;
         }

         delay_us(T_basico);

         if(!RF) {
         buffer|=0x08000000;
         buffer>>=1;
         }
         else buffer>>=1;

         delay_us(T_basico);

         if(!RF){  return FALSE; }
      }

      return TRUE;
   }

   else return FALSE;
}
//==============================================================================
//          Rotina de apagamento dos controles gravados
//==============================================================================


void apagar_eeprom(){
 int8 a;
 for(a=0;a<128;a++){ write_eeprom(a,0xff); }
 tris=tris_led;
 for(a=0;a<6;a++){  LEDG=1; delay_ms(300);LEDG=0;delay_ms(300);}

}
//==============================================================================
//          Rotina de desligamento das saidas acionadas
//==============================================================================

void desliga_saida(){

    if(!--time_led1)
    #ifdef  led1_pulso
     #ifdef led1_invertido
     led1=1;
     #else
     led1=0;
     #endif
    #else
    fled1=0;
    #endif

    if(!--time_led2)
    #ifdef  led2_pulso
     #ifdef led2_invertido
     led2=1;
     #else
     led2=0;
     #endif
    #else
    fled2=0;
    #endif

    if(!--time_led3)
    #ifdef  led3_pulso
     #ifdef led3_invertido
     led3=1;
     #else
     led3=0;
     #endif
    #else
    fled3=0;
    #endif



}
//==============================================================================
//          Rotina de verificao de gravao ou apagamento
//==============================================================================

void testa_lrn(){
 static int8 tempo_led;//contador de apagamento do led prog
 static int1 fx;//flag auxiliar

   fx=LEDG;//copia estado do ledg
 //  tris= tris_lrn;//entrada no pino 7
   if(!LRN&&!fx){
     //tris=tris_led;//saida no pino 7
     tempo_led=0;//zera contador
     LEDG=1;  set_timer0(0);//liga led 'prog', reseta timer( inicio do tempo de programao)
   }
  // tris=tris_led;//saida no pino 7
   if(LEDG&&t0if){//se led prog ligado e houve over flow do timer
     t0if=0;//apaga flag
     tempo_led++;//incrementa tempo de desligamento do led 'prog'
   //  tris= tris_lrn;//entrada no pino 7
     if((tempo_led>apagar)&&!LRN){  apagar_eeprom();}//apagar todos controles
   //  tris=tris_led;//saida no pino 7
     if(tempo_led>T_MAX) LEDG=0; //termina periodo de programao e apaga led prog
   }
   desliga_saida();//desliga saidas ativadas apos tempo
}
//==============================================================================
//          Rotina de pesquisa por controle gravado na eeprom
//==============================================================================
int1 procura(){
  int8 a;
  for(a=0;a<127;a+=3){
    if(make8(buffer,2)!=read_eeprom(a)) continue ;//achou o primeiro byte?
    if(make8(buffer,1)!=read_eeprom(a+1))continue ;//achou o segundo byte?
    if(make8(buffer,0)!=read_eeprom(a+2))continue ;//achou o terceiro byte?
    return true;//sim! Os tres!
  }
  return false;//no!
}

//==============================================================================
//    Rotina para obter o endereo que deve ser gravado o novo controle
//==============================================================================

int8 end_gravar(){
  int8 a;

   for(a=0;a<127;a+=3){
    if(read_eeprom(a)!=0xff) continue ;//byte livre?
    if(read_eeprom(a+1)!=0xff) continue ;//byte livre?
    if(read_eeprom(a+2)!=0xff) continue ;//byte livre?
    return a;//retorna o endereo inicial de 3 bytes livres
   }
   return 0xff;//memria cheia
}

//==============================================================================
//    Rotina de gravao na EEprom do novo controle
//==============================================================================


int8 gravar(){
  int8 adr;
  adr=end_gravar();
  if(!(adr+1)){return 0xff;}//no grava pois memria est cheia
  else{
  write_eeprom(adr,make8(buffer,2));//grava o primeiro byte
  write_eeprom(adr+1,make8(buffer,1));//grava o segundo byte
  write_eeprom(adr+2,make8(buffer,0));//grava o terceiro byte
  return 0x00;//ok
  }
}

//==============================================================================
//          Rotina de tratamento do tx recebido
//==============================================================================

void action(){
 if(procura()) {//se tx foi encontrado na eeprom vai acionar as sadas
 int8 aux;

    aux= make8(buffer,2);//pega os 'botes1' apertados no tx
    if((bit_test(aux,7))&&(bit_test(aux,6))){
       #ifdef led3_pulso
        #ifdef led3_invertido
        led3=0;
        #else
        led3=1;
        #endif
       #else
        if(!fled3){led3=!led3; fled3=1;}
       #endif
       time_led3=v_tempo_acionado;
       }//aciona a sada 3
    else{
     if(bit_test(aux,7)) {
      #ifdef led1_pulso
        #ifdef led1_invertido
        led1=0;
        #else
        led1=1;
        #endif
      #else
       if(!fled1){led1=!led1; fled1=1;}
      #endif
      time_led1=v_tempo_acionado;

      } //aciona a sada 1
     if(bit_test(aux,6)) {
      #ifdef  led2_pulso
         #ifdef led2_invertido
        led2=0;
        #else
        led2=1;
        #endif
      #else
       if(!fled2){led2=!led2; fled2=1;}
      #endif
      time_led2=v_tempo_acionado;
       }//aciona a sada 2
    }
    LEDG=0;
   }
  else{
  // tris=tris_led;
   if(LEDG){
      if(!gravar()) { LEDG=0; }//gravao ok, led learn apaga
   }
  }

}
//==============================================================================
//         Rotina de recepo do tx comparando 2 recepes consecutivas
//==============================================================================


void recebe_tx(){
   static int1 fr;
    if(get_htp()){
      if(!fr) {bufferx=buffer; fr=1;}//salva a primeira recepo
      else{
         fr=0;
         if(bufferx==buffer){ action(); bufferx=0; }//compara a primeira com a segunda
     }
  }
}




//==============================================================================
//          Rotina principal
//==============================================================================


void main() {

   setup_adc_ports(NO_ANALOGS|VSS_VDD);
   setup_adc(ADC_OFF);
   setup_counters(RTCC_INTERNAL,RTCC_DIV_256);
   setup_timer_1(T1_DISABLED);
   setup_comparator(NC_NC_NC_NC);
   setup_vref(FALSE);
   tris=tris_led;
   GPIO=0;
   #ifdef led1_invertido
   led1=1;
   #else
   led1=0;
   #endif
   #ifdef led2_invertido
   led2=1;
   #else
   led2=0;
   #endif
   #ifdef led3_invertido
   led3=1;
   #else
   led3=0;
   #endif

//==============================================================================
//           Loop principal
//==============================================================================

   while(1){

    recebe_tx();//verifica se houve uma recepo de tx
    testa_lrn(); //verifica se foi pressionado boto de aprendizado
   }//while(1)

}//main

